﻿/**
 * Youtube plugin for Showtime  by facanferff (Fábio Ferreira / facanferff.showtime@hotmail.com)
 *
 *  Copyright (C) 2011 facanferff (Fábio Canada / facanferff@hotmail.com)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


(function(plugin) {

    var PREFIX = 'youtube';

    var categories = getCatList('http://gdata.youtube.com/schemas/2007/categories.cat');
    var channelTypes = getCatList('http://gdata.youtube.com/schemas/2007/channeltypes.cat');
    var showGenres = getCatList('http://gdata.youtube.com/schemas/2007/showgenres.cat');
    var movieGenres = getCatList('http://gdata.youtube.com/schemas/2007/moviegenres.cat');

    var service = plugin.createService("Youtube", PREFIX + ":start", "video", true,
	plugin.path + "logo.png");
        
    var settings = plugin.createSettings("Youtube", plugin.path + "logo.png", 
        "Youtube: Video Sharing Service");
    
    settings.createDivider('Browser Settings');
    
    settings.createBool("background", "Enable Background Support", false, function(v) {
        service.background = v;
    });

    settings.createInt("entries", "Maximum number of entries per request (default: 150)", 150, 5, 1000, 5, '', function(v) {
        service.entries = v;
    }); 

    var safeSearch = [
        ['strict', 'Strict'], ['moderate', 'Moderate', true], ['none', 'None']
    ];
    settings.createMultiOpt("safeSearch", "Safe Search", safeSearch, function(v){
        service.safeSearch = v;
    });


    settings.createDivider('General Feed Settings');

    settings.createMultiOpt("category", "Category", categories, function(v){
        service.category = v;
    });
    
    var regions = [
        ['all', 'All', true],
        ['AR', 'Argentina'], ['AU', 'Australia'], ['BR', 'Brazil'],
        ['CA', 'Canada'], ['CZ', 'Czech Republic'], ['FR', 'France'],
        ['DE', 'Germany'], ['GB', 'Great Britain'], ['HK', 'Hong Kong'],
        ['HU', 'Hungary'],
        ['IN', 'India'], ['IE', 'Ireland'], ['IL', 'Israel'],
        ['IT', 'Italy'], ['JP', 'Japan'], ['MX', 'Mexico'],
        ['NL', 'Netherlands'], ['NZ', 'New Zealand'], ['PL', 'Poland'],
        ['RU', 'Russia'], ['ZA', 'South Africa'], ['KR', 'South Korea'],
        ['ES', 'Spain'], ['SE', 'Sweden'], ['TW', 'Taiwan'],
        ['US', 'United States']
    ];
    settings.createMultiOpt("region", "Region", regions, function(v){
        service.region = v;
    });


    settings.createDivider('Channel Feed Settings');

    settings.createMultiOpt("channelType", "Channel Type", channelTypes, function(v){
        service.channelType = v;
    });


    settings.createDivider('Movie Feed Settings');
    
    settings.createMultiOpt("movieGenre", "Genre", movieGenres, function(v){
        service.movieGenre = v;
    });

    var languages = [
        ['all', 'All', true], ['en', 'English'], ['es', 'Spanish'], ['fr', 'French'], ['ja', 'Japanese'], ['pt', 'Portuguese']
    ];
    settings.createMultiOpt("movieLanguage", "Language", languages, function(v){
        service.movieLanguage = v;
    });


    settings.createDivider('Shows Feed Settings');
    
    settings.createMultiOpt("showGenre", "Genre", showGenres, function(v){
        service.showGenre = v;
    });
    
    
    settings.createDivider('Video Settings');
    
    settings.createBool("mode", "Advanced Youtube (Extra video features)", false, function(v) {
        if (v == '1')
            service.mode = 'advanced';
        else
            service.mode = 'simple';
    });
    
    var maximumResolution = [
        ['4', '1080p', true], ['3', '720p'], ['2', '480p'], ['1', '360p'], ['0', '240p']
    ];
    settings.createMultiOpt("maximumResolution", "Highest resolution", maximumResolution, function(v){
        service.maximumResolution = v;
    });

    var minimumResolution = [
        ['4', '1080p'], ['3', '720p'], ['2', '480p'], ['1', '360p'], ['0', '240p', true]
    ];
    settings.createMultiOpt("minimumResolution", "Lowest resolution", minimumResolution, function(v){
        service.minimumResolution = v;
    });
    
    var formats = [
        ['default', 'Default', true], ['mp4', 'MP4'], ['webm', 'WebM'], ['x-flv', 'FLV']
    ];
    settings.createMultiOpt("format", "Video Format", formats, function(v){
        service.format = v;
    });
    
    settings.createBool("subtitles", "Enable Subtitles (powered by Universal Subtitles)", false, function(v) {
        service.subtitles = v;
    });
    
    settings.createDivider('User Settings');
    
    settings.createBool("user", "Enable User Authentication", true, function(v) {
        service.user = v;
    });

    settings.createBool("actions", "Enable User Actions (Add Favorite, Comment, ...)", false, function(v) {
        service.actions = v;
    });
    
    var api = new Youtube_API();
    
    function startPage(page) {
        var v = 3 * 10000000 +  5 * 100000 + 212;
        if (showtime.currentVersionInt < v) {
            page.error('Your version of Showtime is outdated for this plugin. Look at https://www.lonelycoder.com/showtime/download for a new build of it.');
            return;
        }

        page.appendItem(PREFIX + ':browse', 'directory', {title: 'Browse Videos'})
        page.appendItem(PREFIX + ':mixfeeds:'+ 'standard_feeds', 'directory', {title: 'Standard Feeds'})
        page.appendItem(PREFIX + ':mixfeeds:'+ 'channel_feeds', 'directory', {title: 'Channel Feeds'})
        page.appendItem(PREFIX + ':mixfeeds:'+ 'live_feeds', 'directory', {title: 'Live Feeds'})
        page.appendItem(PREFIX + ':mixfeeds:'+ 'movie_feeds', 'directory', {title: 'Movie Feeds'})
        page.appendItem(PREFIX + ':mixfeeds:'+ 'show_feeds', 'directory', {title: 'Show Feeds'})
        page.appendItem(PREFIX + ':mixfeeds:'+ 'trailer_feeds', 'directory', {title: 'Trailer Feeds'})
        
        if (service.user == '1')
            api.login();

        if (api.logged_in())
            page.appendItem(PREFIX + ':user:default', 'directory', {title: 'User Profile'})
    
        page.type = "directory";
        page.contents = "items";
        page.loading = false;

        page.metadata.logo = plugin.path + "logo.png";
        page.metadata.title = "Youtube - Home Page";
    }
  
    plugin.addURI(PREFIX + ":browse", function(page) {
        page.metadata.title = 'Youtube';
        page.metadata.logo = plugin.path + "logo.png";
    
        var categories = getCategoriesList();
    
        for (var i in categories) {
            var entry = categories[i];
            page.appendItem(PREFIX + ':feed:' + escape('https://gdata.youtube.com/feeds/api/videos?category=' + entry[0]), 'directory', {
                title: entry[1]
            })
        }
    
        page.type = "directory";
        page.contents = "items"
        page.loading = false;
    });
  
    plugin.addURI(PREFIX + ":mixfeeds:(.*)", function(page, type) {
        page.metadata.title = 'Youtube';
        page.metadata.logo = plugin.path + "logo.png";
    
        for (var i in api[type]) {
            var entry = api[type][i];
            page.appendItem(PREFIX + ':feed:' + escape(entry[1]),"directory", {
                title: entry[0],
                icon: entry[2]
            });
        }
    
        page.type = "directory";
        page.contents = "items"
        page.loading = false;
    });

    plugin.addURI(PREFIX + ":feed:sort:(.*):(.*)", function(page, category, url) {
        page.type = "directory";
        page.contents = "contents";
        page.loading = false;

        var link = unescape(url);
        var cat = unescape(category);

        var res = getUrlArgs(link);
        link = res.url;

        for (var i in api.orderyby_values) {
            var value = api.orderyby_values[i];
            
            if (cat == "http://gdata.youtube.com/schemas/2007#video" && value[2].indexOf('video') == -1)
                continue;
            if (cat == "http://gdata.youtube.com/schemas/2007#playlist" && value[2].indexOf('playlist') == -1)
                continue;

            res.args['orderby'] = value[1];
            var link_tmp = putUrlArgs(link, res.args);

            page.appendItem(PREFIX + ':feed:' + escape(link_tmp),"directory", {title: value[0]});
        }
    });
  
    function pageController(page, loader) {
        if (service.background == '1')
            page.metadata.background = "http://images2.layoutsparks.com/1/200022/ocean-dreams-blue-waves.jpg";
        page.contents = 'list';
        var offset = 1;
        function paginator() {      
            var num = 0;      
            while(1) {	
                var doc = loader(offset + num).feed;
                page.entries = doc.openSearch$totalResults.$t;
                if (page.entries > service.entries)
                    page.entries = service.entries;
                var c = 0;
                for (var i in doc.entry) {
                    var entry = doc.entry[i];
                    // Override entry.category, needed for Watch History playlist
                    for (var i = 0; i < entry.link.length; i++) {
                        if (entry.link[i].rel == 'http://gdata.youtube.com/schemas/2007#video')
                            entry.category[0].term = 'http://gdata.youtube.com/schemas/2007#video';
                    }
                
                    try {
                        c++;
                        var id, url;
                    
                        if (entry.media$group && entry.media$group.media$content)
                            var duration = showtime.durationToString(entry.media$group.media$content[0].duration);
                        if (entry.yt$statistics) {
                            var views = entry.yt$statistics.viewCount;
                            var favorites = entry.yt$statistics.favoriteCount;
                        }

                        var dateInfo = "";
                        if (entry.published) {
                            var published = getDistanceTime(getTime(entry.published.$t));
                            if (published)
                                dateInfo = 'Published ' + published;
                        }
                    
                        if (entry.updated != undefined) {
                            var updated = getDistanceTime(getTime(entry.updated.$t));
                            if (published)
                                dateInfo += ' | ';
                            if (updated)
                                dateInfo += 'Updated ' + updated;
                        }
                    
                        var titleText = '\n';
                        if (duration)
                            titleText = '<font size="3" color="6699CC"> ( ' + duration + ' )</font>\n'
                        if (views)
                            titleText += '<font size="2" color="66CCFF">Views: ' + views;
                        if (views && favorites)
                            titleText += ' | Favorites: ' + favorites + '</font>\n';
                        else if (favorites)
                            titleText += ' Favorites: ' + favorites + '</font>\n';
                        if (dateInfo)
                            titleText += '<font size="2" color="99CC33">' + dateInfo + '</font>'
                        var title = '<font size="3">' + entry.title.$t + '</font>';
                        
                        var metadataTitle = new showtime.RichText(title + titleText);
                    
                        if (entry.title.$t.toString().indexOf('has added a friend') != -1) {
                            entry.category[0].term = 'http://gdata.youtube.com/schemas/2007#friend'
                        }

                        if (entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#season') {
                            for (var i in entry.gd$feedLink) {
                                var link = entry.gd$feedLink[i];

                                var title = link.rel.valueOf().slice(link.rel.valueOf().indexOf('#')+1).replace(/\./g,' ');

                                title = title.slice(title.indexOf(' ')+1)
                                title = title.charAt(0).toUpperCase() + title.slice(1);

                                title = "Season " + entry.yt$season.season + ' - ' + title;

                                page.appendItem(PREFIX + ':feed:' + escape(link.href), "directory", {                  
                                    title: title
                                });
                            }
                        }
                        else if (entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#video' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#favorite' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#playlist' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#userEvent' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#liveEvent') {
                            id = entry.id.$t.toString().slice(entry.id.$t.toString().lastIndexOf(':')+1)
                            if (id.length > 11) {
                                if (entry.yt$videoid)
                                    id = entry.yt$videoid.$t;
                                else if (entry.content) {
                                    id = entry.content.src.toString().slice(entry.content.src.toString().lastIndexOf('/')+1, 
                                    (entry.content.src.toString().lastIndexOf('?')!=-1)?entry.content.src.toString().lastIndexOf('?'):entry.content.src.toString().length)
                                }
                                else {
                                    id = entry.link[0].href;
                                    id = id.slice(id.indexOf('v=') + 2, id.indexOf('&'));
                                }
                            }
                        
                            page.appendItem(PREFIX + ':video:' + service.mode + ':' + escape(entry.title.$t) + ':' + id,"video", {
                                title: metadataTitle,
                                rating: eval('(entry.gd$rating)?(parseFloat(entry.gd$rating.average) / 5.0):0'),
                                icon: eval('(entry.media$group && entry.media$group.media$thumbnail)?entry.media$group.media$thumbnail[0].url:null'),
                                description: eval('(entry.media$group.media && entry.media$group.media.media$description)?'+
                                'new showtime.RichText(entry.media$group.media$description.$t):null'),
                                duration: eval('(entry.media$group && entry.media$group.yt$duration)?showtime.durationToString(entry.media$group.yt$duration.seconds):null')
                            });
                        }
                        else if (entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#friend' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#subscription') {
                            page.appendItem(PREFIX + ':user:' + entry.yt$username.$t,"directory", {
                                title: new showtime.RichText(entry.title.$t)
                            });
                        }
                        else if (entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#playlistLink' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#show' ||
                        entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#season') {
                            url = entry.content.src;
                            page.appendItem(PREFIX + ':feed:' + escape(url), "directory", {
                                title: new showtime.RichText(entry.title.$t)
                            });
                        }
                        else if (entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#channel') {
                            page.appendItem(PREFIX + ':feed:' + escape(entry.gd$feedLink[0].href), "directory", {
                                title: new showtime.RichText(entry.title.$t)
                            });
                        }
                        else if (entry.category[0].term == 'http://gdata.youtube.com/schemas/2007#channelstandard') {
                            var icon = eval('(entry.media$group)?entry.media$group.media$thumbnail[0].url:null');
                            if (icon && icon.slice(0, 2) == '//') {
                                icon = 'http:' + icon;
                            }

                            page.appendItem(PREFIX + ':user:' + entry.id.$t.toString().slice(entry.id.$t.toString().lastIndexOf(':')+1), "directory", {
                                title: new showtime.RichText(entry.title.$t),
                                icon: icon
                            });
                        }
                    }
                    catch(err) {
                        showtime.trace(err)
                    }
                }
                page.loading = false;	
                num += c;

                if(c == 0 || offset > api.args_common['max-results'] || num > parseInt(service.entries))	  
                    break;
            }
            // Reset arguments for HTTP requests
            api.reset_args();
            offset += num;
        
            return offset < page.entries;    
        }
    
        page.type = "directory";
        paginator();    
        page.paginator = paginator;
    }
  
    plugin.addURI(PREFIX + ":feed:(.*)", function(page, url) {
        var sort_included = false;

        try {
            page.metadata.logo = plugin.path + "logo.png";
            api.reset_args();

            pageController(page, function(offset) { 
                api.args_common['start-index']=offset;
            
                url=unescape(url)
            
                if (url.indexOf('?')!=-1) {
                    var args = url.slice(url.indexOf('?')+1);            
                    args=args.split('&')

                    for (var i in args) {
                        var arg = args[i];
                        var arg_tmp = arg.split('=')
                        api.args_common[arg_tmp[0]]=arg_tmp[1]
                    }
                    url=url.slice(0, url.indexOf('?'))
                }

                var link = putUrlArgs(url, api.args_common);
            
                if (url.indexOf('/standardfeeds/') != -1) {
                    if (service.region && service.region != 'all' && (url.indexOf('/' + service.region + '/') == -1)) {
                        var type_init = url.indexOf('/standardfeeds/');
                        url = url.slice(0, type_init + 15) + service.region + url.slice(type_init + 14);
                    }
                    if (service.category && url.indexOf('_' + service.category) == -1 && service.category != 'all')
                        url += '_' + service.category;
                }
                else if (url.indexOf('api/charts/shows/') != -1) {
                    if (service.showGenre != 'all')
                        api.args_common['genre'] = service.showGenre;
                    if (service.region != 'all')
                        api.args_common['region'] = service.region;
                }
                else if (url.indexOf('api/charts/movies/') != -1) {
                    if (service.movieGenre != 'all')
                        api.args_common['movie-genre'] = service.movieGenre;
                    if (service.region != 'all')
                        api.args_common['region'] = service.region;
                    if (service.movieLanguage != 'all')
                        api.args_common['hl'] = service.movieLanguage;
                    api.args_common['paid-content'] = 'false';
                }
                else if (url.indexOf('api/channelstandardfeeds/') != -1) {
                    if (service.channelType && service.channelType != 'all' && url.indexOf('_' + service.channelType) == -1 && service.channelType != 'all')
                        url += '_' + service.channelType;
                }
        
                var doc = showtime.JSONDecode(showtime.httpGet(url, 
                api.args_common, api.headers_common).toString()); 
                page.metadata.title = doc.feed.title.$t;

                if (!sort_included) {
                    if (doc.feed.category[0].term == "http://gdata.youtube.com/schemas/2007#playlist" ||
                    doc.feed.category[0].term == "http://gdata.youtube.com/schemas/2007#video")
                        page.appendItem(PREFIX + ':feed:sort:' + escape(doc.feed.category[0].term) + ':' + escape(link),"directory", {title: "Sort by..."});

                    sort_included = true;
                }

                return doc;
            });
            page.type = "directory";
        }
        catch (err) {
            if (err == 'Error: HTTP error: 400') {
                var args = '';
                for (var arg in api.args_common)
                    args += '\n' + arg + ': ' + api.args_common[arg]
            
                page.error('The request for the feed contains incompatible args. Please contact facanferff with the following information:\n'+
                err + args);
            }
            else if (err == 'Error: HTTP error: 404')
                page.error('This feed was deleted or is not available at the moment');
            else
                page.error('There was one unknown error...\n' + err);
        }
        page.loading = false;
    });

  
    plugin.addURI(PREFIX + ":user:(.*)", function(page, user) {
        if (!api.logged_in() && user == 'default') {
            showtime.trace('YOUTUBE: User must be authenticated to see this profile');
            page.error('User must be authenticated to see this profile');
            return;
        }

        try {
            api.reset_args();
            var data = showtime.JSONDecode(showtime.httpGet('https://gdata.youtube.com/feeds/api/users/'+user, 
            {alt:'json'}, api.headers_common).toString()).entry;
    
            page.metadata.title = data.title.$t;    
            page.metadata.icon = data.media$thumbnail.url;        
    
            for (var tag in api.user_profile_settings) {
                try {
                    if (tag == 'About me')
                        page.appendPassiveItem("bodytext", new showtime.RichText(eval(api.user_profile_settings[tag])));
                    else
                        page.appendPassiveItem("label", eval(api.user_profile_settings[tag]), {title: tag});
                }
                catch(err) {
                    showtime.trace('Feed doesn\'t contain a tag '+tag+'!');
                }
            }
      
            for (var i in data.gd$feedLink) {
                var link = data.gd$feedLink[i];
                if ((link.rel == 'http://gdata.youtube.com/schemas/2007#user.contacts' ||
                  link.rel == 'http://gdata.youtube.com/schemas/2007#user.subscriptions' ||
                  link.rel == 'http://gdata.youtube.com/schemas/2007#user.newsubscriptionvideos' ||
                  link.rel == 'http://gdata.youtube.com/schemas/2007#user.friendsactivity') &&
                  user != 'default')
                    continue;

                if (user == 'default' && (link.rel == 'http://gdata.youtube.com/schemas/2007#user.liveevent' ||
                    link.rel == 'http://gdata.youtube.com/schemas/2007#user.recentactivity'))
                    continue;
              
                if (link.rel != 'http://gdata.youtube.com/schemas/2007#user.inbox') {
                    var title = link.rel.valueOf().slice(link.rel.valueOf().indexOf('#')+1).replace(/\./g,' ');
                    title = title.slice(title.indexOf(' ')+1)
                    title = title.charAt(0).toUpperCase() + title.slice(1)
                  
                    page.appendAction("navopen", PREFIX + ':feed:'+escape(link.href), true, {                  
                        title: title
                    });
                }
            }

            page.appendAction("navopen", PREFIX + ':feed:'+escape('https://gdata.youtube.com/feeds/api/users/' + user + '/shows'), true, {                  
                title: 'Shows'
            });

            if (api.logged_in() && user != 'default' && service.actions == '1') {
                page.appendAction("pageevent", "addcontact", true, {title:'Add Contact'})
                page.appendAction("pageevent", "subscribeUser", true, {title:'Subscribe User'})
                page.appendAction("pageevent", "subscribeChannel", true, {title:'Subscribe Channel'})
            
                page.onEvent('subscribeUser', function() {
                    api.subscribe(eval(api.user_profile_settings.Username), "user")
                })
            
                page.onEvent('subscribeChannel', function() {
                    api.subscribe(eval(api.user_profile_settings.Username), "channel")
                })
            }
    
            page.type = "item";
    
        }
        catch(err) {
            if (err == 'Error: HTTP error: 404')
                page.error('The user specified does not exist. \nReturn to the previous page please.')
            else
                page.error('There was one unknown error...');
        }
    
        page.loading = false;
        page.metadata.title = 'Youtube - User Screen';
    });
  
    function getVideosList(page, id, number_items) {
        var data = showtime.httpGet('http://www.youtube.com/watch?v='+id, null, api.headers_common).toString();
        
        if (data.indexOf('<div id="watch-player" class="flash-player" style="background: none repeat scroll 0% 0% transparent;">') == -1 && 
            data.indexOf('<div id="watch-player" class="unavailable-player">') != -1) {
                data = data.replace(/\n/g, '')
                var error = data.replace(/\n/, '').match('<div class="yt-alert-message">(.+?)</div>');
                
                return error[1];
            }
        
        var videos = unescape(data.slice(data.indexOf('"url_encoded_fmt_stream_map": "url=')+35,
        data.indexOf('",', data.indexOf('"url_encoded_fmt_stream_map": "url=')))).split(',url=');
    
        var quality_added = [];
        var videos_list_tmp = [];
        var videos_list = [];
    
        // Get each video link available
        for (var i in videos) {
            var vid = videos[i];
            var url = vid.slice(0, vid.indexOf('\\u0026quality'))
        
            var format = vid.match('type=video/([^;|\\u0026]+)')[1];
            if (format.indexOf('\\u0026') != -1)
                format = format.slice(0, format.indexOf('\\u0026'));
        
            var quality = vid.slice(vid.indexOf('quality=')+8, vid.indexOf('\\u0026', vid.indexOf('quality=')))
            var video_item = {
                video_url: escape(url),
                quality: quality,
                format: format
            }
        
            videos_list_tmp.push(video_item);
        }
    
        var resolutions = [
        'small',
        'medium',
        'large',
        'hd720',
        'hd1080'
        ]
    
        var items = 0;
        for (var i in videos_list_tmp) {
            var item = videos_list_tmp[i];
            if (items >= parseInt(number_items))
                break;
        
            var j = 0;
            for (var i = 0; i < resolutions.length; i++) {
                if (item.quality == resolutions[i]) {
                    j = i;
                    break;
                }
            }
            if (service.maximumResolution && j > parseInt(service.maximumResolution) || quality_added.indexOf(item.quality)!=-1)
                continue;

            if (service.minimumResolution && j < parseInt(service.minimumResolution))
                break;
        
            var video_item_tmp = {};
        
            var formatVar = 'default';
            if (service.format)
                formatVar = service.format;
        
            if ((formatVar != 'default' && formatVar == item.format) || formatVar == 'default') {
                video_item_tmp = {
                    video_url: escape(item.video_url),
                    quality: item.quality,
                    format: item.format
                }
                videos_list.push(video_item_tmp);
                quality_added.push(item.quality);
                items++;
            }
        }
    
        return videos_list;
    }
  
    function playVideo(page, title, id, video_url) {
        var videoParams = {      
            title: unescape(unescape(title)),
            sources: [{	
                url: unescape(unescape(video_url))   
            }]    
        }
        if (service.subtitles == '1')
            videoParams.subtitles = getSubtitles(id);
    
        page.source = "videoparams:" + showtime.JSONEncode(videoParams);
        page.type = "video";
    }
  
    plugin.addURI(PREFIX + ":video:simple:(.*):(.*)", function(page, title, id) {
        try {
            var video_url = getVideosList(page, id, 1);

            if (typeof(video_url) == "string") {
                page.error(video_url);
            }

            video_url = video_url[0].video_url;
        }
        catch (err) {
            page.error(err);
            return;
        }
    
        page.loading = false;
        playVideo(page, title, id, video_url);
    });
  
    plugin.addURI(PREFIX + ":video:advanced:(.*):(.*)", function(page, title, id) {
        var video = showtime.JSONDecode(showtime.httpGet('https://gdata.youtube.com/feeds/api/videos/' + id, {
            'alt' : 'json'
        }, api.headers_common).toString()).entry;
    
        page.metadata.title = unescape(title);
        page.metadata.icon = video.media$group.media$thumbnail[0].url;

        if (service.background == '1')
            page.metadata.background = "http://www.free-desktop-backgrounds.net/free-desktop-wallpapers-backgrounds/free-hd-desktop-wallpapers-backgrounds/349422168.jpg";
    
        for (var i in api.video_settings) {
            var element = api.video_settings[i];
            try {
                if (element[0] == 'Rating')
                    page.appendPassiveItem("rating", eval(element[1]));
                else
                    page.appendPassiveItem("label", eval(element[1]), {title: element[0]}); 
            }
            catch(err) {
                showtime.trace('Video '+id+' doesn\'t have a '+element[0]+' tag!');
            }
        }
    
        page.appendPassiveItem("divider");  
    
        page.appendPassiveItem("bodytext", new showtime.RichText(video.media$group.media$description.$t));
    
        var author = video.author[0].name.$t;
        page.appendAction("navopen", PREFIX + ':user:'+author, true, {                  
            title: author     
        });

        var videos_list = getVideosList(page, id, 'any');

        if (typeof(videos_list) == "string") {
            page.error(videos_list);
        }
        else {
            for (var i in videos_list) {
                var item = videos_list[i];
                page.appendAction("navopen", PREFIX + ':video:stream:'+escape(title)+':'+escape(id) + ':' + item.video_url, true, {      
                    title: item.quality
                });
            }
    
            for (var i in video.link) {
                var link = video.link[i];
                if (link.rel == 'http://gdata.youtube.com/schemas/2007#video.related') {
                    page.appendAction("navopen", PREFIX + ':feed:'+escape('https://gdata.youtube.com/feeds/api/videos/'+id+'/related'), true, {                  
                        title: 'Related videos'          
                    });
                }
                else if (link.rel == 'http://gdata.youtube.com/schemas/2007#video.responses') {
                    page.appendAction("navopen", PREFIX + ':feed:'+escape('https://gdata.youtube.com/feeds/api/videos/'+id+'/responses'), true, {                  
                        title: 'Responses videos'          
                    });
                }
            }
            if (api.logged_in() && service.actions == '1') {
                page.appendAction("pageevent", "like", true, {title:'Like'})
                page.appendAction("pageevent", "dislike", true, {title:'Dislike'})
                page.appendAction("pageevent", "comment", true, {title:'Comment'});
                page.appendAction("pageevent", "addFavorite", true, {title:'Add favorite'});
                page.appendAction("pageevent", "watchLater", true, {title:'Watch Later'});
        
                page.onEvent('like', function() {
                    api.like(id, 'like') 
                });
        
                page.onEvent('dislike', function() {
                    api.like(id, 'dislike') 
                });
        
                page.onEvent('comment', function() {
                    api.comment(id) 
                });
        
                page.onEvent('addFavorite', function() {
                    api.addFavorite(id) 
                });
        
                page.onEvent('watchLater', function() {
                    api.watchLater(id) 
                });
            }
    
            page.metadata.logo = plugin.path + "logo.png";
            page.type = "item";
        }
    
        page.loading = false;
    });
  
    // We need to use this function so we can pass the correct title of video
    plugin.addURI(PREFIX + ":video:stream:(.*):(.*):(.*)", function(page, title, id, video_url) {
        playVideo(page, title, id, video_url);
    });
  

    function getCatList(link) {
        var atom = new Namespace("http://www.w3.org/2005/Atom");
      
        var data = new XML(valid_xml(showtime.httpGet(link).toString()));
        var list = [];
        list.push(['all', 'All', true]);
      
        for (var i in data.atom::category) {
            var entry = data.atom::category[i];
            var item = [entry.@term, entry.@label]
            list.push(item)
        }
      
        return list;
    }
  
    //workaround for "Syntax Error: xml is a reserved identifier" - From Andreus Sebes's WebMedia
    function valid_xml(xmltext)
    {
        xmltext=xmltext.replace(/^[\s\S]*?(<[^\?!])/, "$1");

        return xmltext;
    }
  
    function getTime(date) {
        var time = date.match('(.*)-(.*)-(.*)T(.*):(.*):(.*)..*Z');
        if (time) {
            var dateVar = new Date(time[1], time[2], time[3], time[4], time[5], time[6]);
            return dateVar;
        }
        return -1;
    }
  
    function getDistanceTime(date) {
        if (date == -1)
            return null;

        var today = new Date();
        var years_past = today.getFullYear() - date.getFullYear();
        var months_past = (today.getMonth() + 1) - date.getMonth();
        var days_past = today.getDate() - date.getDate();
        var hours_past = today.getHours() - date.getHours();
        var minutes_past = today.getMinutes() - date.getMinutes();
        var seconds_past = today.getSeconds() - date.getSeconds();
      
        if (years_past >= 1) {
            if (months_past < 0)
                return (12 + months_past) + " months ago";
            else
                return years_past + " years ago";
        }
        else if ( years_past < 1 && months_past > 0) {
            if (months_past == 1)
                return "last month";
            return months_past + " months ago";
        }
        else if (days_past > 0 && months_past == 0) {
            if (days_past == 1)
                return "Yesterday";
            return days_past + " days ago";
        }
        else if (hours_past > 0 && days_past == 0) {
            if (hours_past == 1)
                return hours_past + " hour ago"; 
            return hours_past + " hours ago";
        }
        else if (hours_past == 0 && days_past == 0) {
            if (minutes_past == 1)
                return minutes_past + " minute ago";
            return minutes_past + " minutes ago";
        }
        else if (seconds_past > 0 && minutes_past < 0)
            return seconds_past + " seconds ago";
      
        return date;
    }
  
  
/*------------------------------------------------------------------------------
 * Functions for Youtube API
 -----------------------------------------------------------------------------*/
    function Youtube_API() {
        // Login variables
        this.credentials = {};
        this.SID = '';

        this.logged_in = function() {
            if (service.user == '1' && this.headers_common.Authorization && this.headers_common.Authorization > 'GoogleLogin ') {
                showtime.trace('YOUTUBE: User is authenticated');
                return true;
            }
            else { 
                showtime.trace('YOUTUBE: User is not authenticated');
                return false;
            }
        }

        this.login = function() {
            if(this.credentials.username)     
                return true;
    
            var reason = "Login to Youtube account";    
            var do_query = false; 
            while(1) {               
                this.credentials = plugin.getAuthCredentials("Youtube - Video sharing service",	
            reason, do_query, 'api', false); 
        
                if(!this.credentials) {	
                    if(!do_query) {	  
                        do_query = true;	  
                        continue;	
                    }	
                    return false;      
                }
                if (this.credentials.rejected)
                    return false;
                try {
                    var v = showtime.httpPost("https://www.google.com/accounts/ClientLogin", {	
                        'Email' : this.credentials.username,	
                        'Passwd' : this.credentials.password,
                        'source' : 'showtime'
                    }, {
                        'service' : 'youtube'
                    }, this.headers_common);
                    v = v.toString().split('\n');
                    this.headers_common.Authorization = 'GoogleLogin ' + v[2];

                    showtime.trace('YOUTUBE: Logged in as user ' + this.credentials.username);
                }
                catch(err) {
                    reason = err;
                    do_query = true;
                    showtime.trace('YOUTUBE: Failed to log in, ' + err);
                    continue;
                }
                return true;    
            }
        };
    
        // Standard feeds
        this.standard_feeds = [
        ['Top Rated', 'https://gdata.youtube.com/feeds/api/standardfeeds/top_rated', 'http://i1163.photobucket.com/albums/q552/facanferff/top_rated.png'],
        ['Top Favorites', 'https://gdata.youtube.com/feeds/api/standardfeeds/top_favorites', 'http://i1163.photobucket.com/albums/q552/facanferff/favorites.png'],
        ['Most Viewed', 'https://gdata.youtube.com/feeds/api/standardfeeds/most_viewed', 'http://i1163.photobucket.com/albums/q552/facanferff/most_viewed.jpg'],
        ['Most Shared', 'https://gdata.youtube.com/feeds/api/standardfeeds/most_shared', 'http://i1163.photobucket.com/albums/q552/facanferff/most_shared.png'], // Experimental feature
        ['Most Popular', 'https://gdata.youtube.com/feeds/api/standardfeeds/most_popular', 'http://i1163.photobucket.com/albums/q552/facanferff/most_popular.png'],
        ['Most Recent', 'https://gdata.youtube.com/feeds/api/standardfeeds/most_recent', 'http://i1163.photobucket.com/albums/q552/facanferff/recent.png'],
        ['Most Discussed', 'https://gdata.youtube.com/feeds/api/standardfeeds/most_discussed', 'http://i1163.photobucket.com/albums/q552/facanferff/most_discussed.png'],
        ['Most Responded', 'https://gdata.youtube.com/feeds/api/standardfeeds/most_responded', 'http://i1163.photobucket.com/albums/q552/facanferff/most_responded.png'],
        ['Recently Featured', 'https://gdata.youtube.com/feeds/api/standardfeeds/recently_featured', 'http://i1163.photobucket.com/albums/q552/facanferff/featured.png'],
        ['Trending videos', 'https://gdata.youtube.com/feeds/api/standardfeeds/on_the_web', 'http://i1163.photobucket.com/albums/q552/facanferff/trending.png'] // Experimental feature
        ];
    
        // Channel feeds
        this.channel_feeds = [
        ['Most viewed', 'https://gdata.youtube.com/feeds/api/channelstandardfeeds/most_viewed', 'http://i1163.photobucket.com/albums/q552/facanferff/most_viewed.jpg'],
        ['Most subscribed', 'https://gdata.youtube.com/feeds/api/channelstandardfeeds/most_subscribed', 'http://i1163.photobucket.com/albums/q552/facanferff/favorites.png']
    ];

        // Live feeds
        this.live_feeds = [
        ['Featured', 'https://gdata.youtube.com/feeds/api/charts/live/events/featured', 'http://i1163.photobucket.com/albums/q552/facanferff/featured.png'],
        ['Live Now', 'https://gdata.youtube.com/feeds/api/charts/live/events/live_now', 'http://i1163.photobucket.com/albums/q552/facanferff/favorites.png'],
        ['Upcoming', 'https://gdata.youtube.com/feeds/api/charts/live/events/upcoming', 'http://i1163.photobucket.com/albums/q552/facanferff/trending.png'],
        ['Recently Broadcasted', 'https://gdata.youtube.com/feeds/api/charts/live/events/recently_broadcasted', 'http://i1163.photobucket.com/albums/q552/facanferff/recent.png']
        ];
    
        // Movie feeds
        this.movie_feeds = [
        //['Featured', 'https://gdata.youtube.com/feeds/api/charts/movies/featured', 'http://i1163.photobucket.com/albums/q552/facanferff/featured.png'],
        ['Most Popular', 'https://gdata.youtube.com/feeds/api/charts/movies/most_popular', 'http://i1163.photobucket.com/albums/q552/facanferff/most_popular.png'],
        ['Most Recent', 'https://gdata.youtube.com/feeds/api/charts/movies/most_recent', 'http://i1163.photobucket.com/albums/q552/facanferff/recent.png'],
        ['Trending', 'https://gdata.youtube.com/feeds/api/charts/movies/trending', 'http://i1163.photobucket.com/albums/q552/facanferff/trending.png']
        ];

        // Show feeds
        this.show_feeds = [
        ['Most Popular', 'https://gdata.youtube.com/feeds/api/charts/shows/most_popular', 'http://i1163.photobucket.com/albums/q552/facanferff/most_popular.png'],
        ['Latest', 'https://gdata.youtube.com/feeds/api/charts/shows/latest', 'http://i1163.photobucket.com/albums/q552/facanferff/recent.png']
        ];
    
        // Trailer feeds
        this.trailer_feeds = [
        ['Most Popular', 'https://gdata.youtube.com/feeds/api/charts/trailers/most_popular', 'http://i1163.photobucket.com/albums/q552/facanferff/most_popular.png'],
        ['Most Recent', 'https://gdata.youtube.com/feeds/api/charts/trailers/most_recent', 'http://i1163.photobucket.com/albums/q552/facanferff/recent.png']
        ];
    
        this.user_profile_settings = {
            'Username':'data.yt$username.$t',
            'First name':'data.yt$firstName.$t',
            'Last name':'data.yt$lastName.$t',
            'Age':'data.yt$age.$t',
            'Gender':'data.yt$gender.$t',
            'Location':'data.yt$location.$t',
            'Hometown':'data.yt$hometown.$t',
            'Company':'data.yt$company.$t',
            'Occupation':'data.yt$occupation.$t',
            'School':'data.yt$school.$t',
            'About me':'data.yt$aboutMe.$t',
            'Hobbies':'data.yt$hobbies.$t',
            'Movies':'data.yt$movies.$t',
            'Music':'data.yt$music.$t',
            'Books':'data.yt$books.$t',
            'Videos uploaded':'data.yt$channelStatistics.videoCount.$t',
            'Channel viewed':'data.yt$channelStatistics.viewCount.$t',
            'Subscribers':'data.yt$statistics.subscriberCount.$t'
        };
    
        this.video_settings = [
        ['Category', 'video.media$group.media$category[0].$t'],
        ['Rating', 'parseFloat(video.gd$rating.average/5)'],
        ['Views', 'data.yt$statistics.viewCount'],
        ['Duration', 'showtime.durationToString(data.media$group.yt$duration.seconds)']
        ];

        this.orderyby_values = [
        ['Published', 'published', 'video/playlist'],
        ['Relevance', 'relevance', 'video'],
        ['Views', 'viewCount', 'video/playlist'],
        ['Rating', 'rating', 'rating'],
        ['Position', 'position', 'playlist'],
        ['Comments', 'commentCount', 'playlist'],
        ['Duration', 'duration', 'playlist'],
        ['Title', 'title', 'video/playlist']
        ];
    
        this.reset_args = function() {
            this.args_common = {
                'alt' : 'json',
                'max-results' : 25,
                'safeSearch' : (service.safeSearch)?service.safeSearch: 'moderate'
            }
        };
        this.reset_args();
    
        // Headers for HTTP requests
        this.headers_common = {
            'GData-Version' : '2.1',
            'X-GData-Key' : 'key=AI39si7gfa8PEGC6qMb5Kk04aPInFlZVRIPZio6fNE9-0uwS4Qvo9dbhGxzeWIEQ8J4hMHGMtw2xOHuDGn3ped2EktTAVqCU9w' //Don't steal API key
        }
    
        // Like a video given its id
        this.like = function(id, type) {
            var request = '<?xml version="1.0" encoding="UTF-8"?>'+
            '<entry xmlns="http://www.w3.org/2005/Atom" '+
            'xmlns:yt="http://gdata.youtube.com/schemas/2007">'+
            '<yt:rating value="'+type+'"/></entry>';
        
            this.headers_common["Content-type"] = "application/atom+xml";
        
            try {
                var data = showtime.httpPost("http://gdata.youtube.com/feeds/api/videos/"+id+"/ratings", request, null, this.headers_common);
                showtime.message(type.charAt(0).toUpperCase() + type.slice(1) + ' video succesfully', true, false);
            }
            catch(ex) {
                showtime.message('Error while trying to ' + type + ' video: ' + ex, true, false);
            }
        }
    
        // Comment a video given its id
        this.comment = function(id) {
            var search = showtime.textDialog('Your comment to the video: ', true, true);
            if (search.rejected)
                return -1 //canceled
        
            var request = '<?xml version="1.0" encoding="UTF-8"?>'+
            '<entry xmlns="http://www.w3.org/2005/Atom" '+
            'xmlns:yt="http://gdata.youtube.com/schemas/2007">'+
            '<content>'+search.input+'</content>'+
            '</entry>';
        
            this.headers_common["Content-type"] = "application/atom+xml";
        
            try {
                var data = showtime.httpPost("http://gdata.youtube.com/feeds/api/videos/"+id+"/comments", request, null, this.headers_common);
                showtime.message('Commented video succesfully', true, false);
            }
            catch(ex) {
                showtime.message('Error while trying to comment video: ' + ex, true, false);
            }

            return 0;
        }
    
        this.addFavorite = function(id) {
            var request = '<?xml version="1.0" encoding="UTF-8"?>'+
            '<entry xmlns="http://www.w3.org/2005/Atom">'+
            '<id>'+id+'</id>'+
            '</entry>';
        
            this.headers_common["Content-type"] = "application/atom+xml";
        
            try {
                var data = showtime.httpPost("http://gdata.youtube.com/feeds/api/users/default/favorites", request, null, this.headers_common);
                showtime.message('Added video to Favorites succesfully', true, false);
            }
            catch(ex) {
                showtime.message('Error while trying to add video to Favorites: ' + ex, true, false);
            }
        }
    
        this.watchLater = function(id) {
            var request = '<?xml version="1.0" encoding="UTF-8"?>'+
            '<entry xmlns="http://www.w3.org/2005/Atom" '+
            'xmlns:yt="http://gdata.youtube.com/schemas/2007">'+
            '<id>'+id+'</id>'+
            '<yt:position>1</yt:position>'+
            '</entry>';
        
            this.headers_common["Content-type"] = "application/atom+xml";
        
            try {
                var data = showtime.httpPost("http://gdata.youtube.com/feeds/api/users/default/watch_later", request, null, this.headers_common);
                showtime.message('Added video to Watch Later succesfully', true, false);
            }
            catch(ex) {
                showtime.message('Error while trying to add video to Watch Later: ' + ex, true, false);
            }
        }
    
        this.addContact = function(id) {
            var request = '<?xml version="1.0" encoding="UTF-8"?>'+
            '<entry xmlns="http://www.w3.org/2005/Atom" '+
            'xmlns:yt="http://gdata.youtube.com/schemas/2007">'+
            '<yt:username>'+id+'</yt:username>'+
            '</entry>';
        
            this.headers_common["Content-type"] = "application/atom+xml";
        
            try {
                var data = showtime.httpPost("http://gdata.youtube.com/feeds/api/users/default/contacts", request, null, this.headers_common);
                showtime.message('Added contact succesfully', true, false);
            }
            catch(ex) {
                showtime.message('Error while trying to add contact: ' + ex, true, false);
            }
        }
    
        this.subscribe = function(id, term) {
            var request = '<?xml version="1.0" encoding="UTF-8"?>'+
            '<entry xmlns="http://www.w3.org/2005/Atom" '+
            'xmlns:yt="http://gdata.youtube.com/schemas/2007">'+
            '<category scheme="http://gdata.youtube.com/schemas/2007/subscriptiontypes.cat" '+
            'term="'+term+'"/>'+
            '<yt:username>'+id+'</yt:username>'+
            '</entry>';
        
            this.headers_common["Content-type"] = "application/atom+xml";
        
            try {
                var data = showtime.httpPost("http://gdata.youtube.com/feeds/api/users/default/subscriptions", request, null, this.headers_common);
                showtime.message('Subscribed to ' + term + ' succesfully', true, false);
            }
            catch(ex) {
                showtime.message('Error while trying to subscribe to ' + term + ': ' + ex, true, false);
            }
        }
    }

    function getSubtitles(video_id) {
        var args = 'video_url=' + escape('http://youtube.com/watch?v=' + video_id);
        var data = showtime.httpPost('http://www.universalsubtitles.org/en/videos/create/', args).toString();
    
        var begin = data.indexOf('<ul id="subtitles-menu">');
        var end = data.indexOf('</ul>', begin);
        var nice = data.slice(begin, end);
        var split = nice.split('</li>');
    
        var subtitles = [];
        
        for (var i in split) {
            var item = split[i];
            var language = item.match('<a href="/en/videos/(.*)/(.*)/(.*)/">');
            if (!language) {
                language = item.match('<a href="/en/videos/(.*)/">');
                if (!language || item.indexOf('<span class="done_percentage">(0 Lines)</span>') != -1)
                    continue;
                subtitles.push({
                    url: 'http://www.universalsubtitles.org/widget/download_srt/?video_id=' + language[1],
                    language: getValue(item, '<span class="done_indicator"></span>', '<span class="done_percentage">').replace(/\n/g, '').replace(/ /g, '')
                });
            }
            else {
                if (item.indexOf('<span class="done_percentage">(0 Lines)</span>') != -1)
                    continue;
                subtitles.push({
                    url: 'http://www.universalsubtitles.org/widget/download_srt/?video_id=' + language[1] + '&lang_pk=' + language[3],
                    language: language[2]
                });
            }
        }
        return subtitles;
    }

    function getValue(url, start_string, end_string)
    {
        var begin_temp = url.indexOf(start_string) + start_string.toString().length;
        var end_temp = url.indexOf(end_string, begin_temp);
		
        var string = url.slice(begin_temp, end_temp);
        return unescape(string);
    }

    function getUrlArgs(url) {
        var link = url;

        var result = {
            url: link,
            args: {}
        };

        var args = {};

        if (link.indexOf('?') != -1) {
            var args_tmp = url.slice(url.indexOf('?') + 1);            
            args_tmp = args_tmp.split('&');

            for (var i in args_tmp) {
                var arg = args_tmp[i];
                var arg_tmp = arg.split('=');
                args[arg_tmp[0]] = arg_tmp[1];
            }

            link = link.slice(0, link.indexOf('?'));
        }

        result.url = link;
        result.args = args;
        return result;
    }

    function putUrlArgs(url, args) {
        var link = url + '?';
        var args_end = false;
        
        for (var i in args) {
            link += i + '=' + args[i] + '&';
            args_end = true;
        }

        if (args_end)
            link = link.slice(0, link.length - 1);

        return link;
    }
  
    plugin.addURI(PREFIX+":start", startPage);

    plugin.addSearcher("Youtube - Videos", plugin.path + "logo.png",    
    function(page, query) { 
        try {
            pageController(page, function(offset) {	
                api.args_common['start-index'] = offset;
                api.args_common.q = query;

                page.appendItem(PREFIX + ':feed:sort:' + escape("http://gdata.youtube.com/schemas/2007#video") + ':' + escape("https://gdata.youtube.com/feeds/api/videos/?q=" + query),"directory", {title: "Sort by..."});
            
                return showtime.JSONDecode(showtime.httpGet("https://gdata.youtube.com/feeds/api/videos", 
                    api.args_common, api.headers_common).toString());
            });
        }
        catch(err){
            showtime.trace('Search Youtube - Videos: '+err)
        }
    });

    plugin.addSearcher("Youtube - Playlists", plugin.path + "logo.png",    
    function(page, query) { 
        try {
            pageController(page, function(offset) {	
                api.args_common['start-index'] = offset;
                api.args_common.q = query;

                var doc = showtime.JSONDecode(showtime.httpGet("https://gdata.youtube.com/feeds/api/playlists/snippets",
                    api.args_common, api.headers_common).toString());
            });
        }
        catch(err){
            showtime.trace('Search Youtube - Playlists: '+err)
        }
    });

    plugin.addSearcher("Youtube - Channels", plugin.path + "logo.png",    
    function(page, query) {
        try {
            pageController(page, function(offset) {	
                api.args_common['start-index'] = offset;
                api.args_common.q = query;
                return showtime.JSONDecode(showtime.httpGet("https://gdata.youtube.com/feeds/api/channels",
                    api.args_common, api.headers_common).toString());
            });
        }
        catch(err){
            showtime.trace('Search Youtube - Channels: '+err)
        }
    });

})(this);
